(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     13820,        375]*)
(*NotebookOutlinePosition[     14603,        402]*)
(*  CellTagsIndexPosition[     14559,        398]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["A Simple Boundary Generator ", "Title"],

Cell["P.-A. Chevalier, december 1999", "SmallText"],

Cell[CellGroupData[{

Cell[TextData[{
  "This Notebook makes it possible to create the geometry of a boundary, and \
to save the corresponding data directly in a file in the special EasyMesh \
format. The filename can be defined by user, but its extension must be ",
  StyleBox["\"*.d\"",
    FontWeight->"Bold"],
  "  for compatibility with EasyMesh. "
}], "Subsubtitle"],

Cell[CellGroupData[{

Cell["Instructions and example:", "Section"],

Cell[TextData[{
  "Suppose you want to use the program \"EasyMesh\" to generate a mesh on a \
particular given 2D domain, for example in a FEM problem. The information \
Easymesh needs the know is the exact description of the boundary of the \
domain, in a specially structured datafile. This program help you to create \
the boundary and the corresponding datafile, under the following assumptions:\
\n\n1) the domain is simply connected and not too much complicated\n2) the \
boundary consists of a set of closed smooth curves (smoothness can be relaxed \
in special cases)\n3) all what you do must be coherent !\n\nTo achieve your \
work, you have to do the following steps:\n\n1) First, give a name for the \
outputfile and a short header which will appear as a comment at the beginning \
of \n    the file.\n    The extension of the filename must be \".d\". \n    \
For example:\n\t\t",
  StyleBox["filename = \"test6.d\" ;\n\t\theader = \"test6.d\";\n\t\t",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "\n2) Secondly, you have to assign the variable  ",
  StyleBox["npieces",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "  the total number of separated closed curves which\n    compose the \
complete boundary:\n \t\t",
  StyleBox["npieces = 2; ",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "\n \t\n 3) For each closed curve (that is: for each ",
  StyleBox["k = 1 to npieces",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "), you define the 4 following variables: \n \n \t\t",
  StyleBox["pts[k]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "  \t: contains the list of xy-coordinates of the successive points which \
compose the kth\n \t\t\t\t closed curve, in a logical order ;\n \t\t          \
         Attention: the curve must be closed, but the end point (= first \
point) must be given \n \t\t                   only once in the list !\n \t\t \
                  The points have to be numbered anticlockwise for an \
external boundary of the domain\n \t\t                   and clockwise for an \
internal boundary (for example: a hole).\n\n\t\t",
  StyleBox["conditions[k]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " : for each pair of successive points in the curve:  give 1 for a \
Dirichlet condition or \n\t\t\t\t2 for a Neuman condition. The length of the \
list must be the same that for ",
  StyleBox["pts[k]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ". \n\t\t\n\t\t",
  StyleBox["raffinment[k]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ": give here a number corresponding to the raffinment of the mesh you want \
in the \n\t\t\t\tneighborhood of the point (see EasyMesh for more \
informations).\n\t\t\n\t\t",
  StyleBox["marker[1]: ",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "give here any special marker for the corresponding piece of curve.\n\n\t\
Example: we construct here the boundary of a simple rectangular domain with a \
circular hole:\n\t\t\n\t\t",
  Cell[BoxData[
      \(\(pts[1] = \ {{0, 0}, {2, 0}, {2, 1}, {0, 1}}\ ; \)\)], "Input"],
  "\n\t\t",
  Cell[BoxData[
      \(\(conditions[1]\  = \ {2, 1, 2, 1}\ ; \)\)], "Input"],
  "\n\t\t",
  Cell[BoxData[
      \(raffinment[1]\  = \ 0.05; \nmarker[1]\  = \ 1\ ; \)], "Input"],
  "\n\n",
  StyleBox[
  "\t\tnphi = 10 ;\n\t\tdphi = 2 Pi/nphi ;\n\t\tpts[2]= Table[{1 + 0.3 \
Cos[-dphi k], 0.5 + 0.3 Sin[-dphi k]}, {k,0,nphi-1}] ;\n\t\tconditions[2] = \
Table[2, {k,0,nphi-1}] ;\n\t\traffinment[2] = 0.05;\n\t\tmarker[2] = 2 ;\n\n\n\
",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "4) Then, you only have to execute the three following instructions:\n\n\t- \
   ",
  StyleBox["assembleboundary",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "  generates a list  ",
  StyleBox["bpoints",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["bndry",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " which contain the informations of the\n\t\t\t\t boundary ;\n\t-    ",
  StyleBox["showboundary[bpoints, bndry]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "  shows you a graphic of the boundary you have defined ;\n\t-    ",
  StyleBox["createfile[bpoints, bndry, filename, header]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " creates the outputfile for EasyMesh.\n\t\n\t\nThats All. Have a nice Work \
!"
}], "Text",
  Editable->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Initializations", "Section",
  InitializationCell->True],

Cell[BoxData[
    \(Off[General::spell1]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(createPointsList[pts_List, raff_, marker_]\  := \ \ \n\t\ \ 
      Module[{length, npts}, \n\t\t\ \ length = Length[pts]\ ; \n\t\ \ \ \ 
        npts\  = 
          Table[{\(pPointer++\), pts[\([k]\)], raff, marker}, 
            \ {k, 1, length}]\ ; \n\t\tReturn[npts]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(createSegmentsList[newpts_List, bcond_List]\  := \n\t\ \ 
      Module[{length, sgts}, \n\t\t\ length\  = \ Length[newpts]\ ; \n\t\t\ 
        sgts\  = \ 
          Table[{{newpts[\([k, 1]\)], newpts[\([k + 1, 1]\)]}, 
              bcond[\([k]\)]}, {k, 1, length - 1}]\ ; \n\ \ \ \ \ \ 
        sgts\  = \ 
          Append[sgts, 
            \ {{newpts[\([length, 1]\)], newpts[\([1, 1]\)]}, 
              bcond[\([length]\)]}]\ ; \n\t\ Return[sgts]\ \n\t\t]\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(\(createfile[bpoints_List, bndry_List, filename_String, header_String]
        \  := \ \n\t
      Module[{npoints, nsegments, real2string, int2string, n, l1, str1, k, 
          outstream}, \nnpoints\  = \ \ Length[bpoints]\ ; \n
        nsegments\  = \ \ Length[bndry]\ ; \nl1\  = \ StringLength[header]\ ; 
        \nstr1\  = \ "\<\>"\ ; \n
        Do[str1\  = \ StringJoin[str1, "\<#\>"], {k, 1, l1 + 4}]\ ; \n
        str1\  = \ StringJoin[str1, "\<\n\>"]\ ; \t\t\n
        real2string[n_]\  := \ ToString[PaddedForm[n, {4, 2}]]\ ; \n
        int2string[n_]\  := \ ToString[PaddedForm[n, 2]]\ ; \n
        outstream = OpenWrite[filename]; \nWriteString[outstream, str1]\ ; \n
        WriteString[outstream, "\<# \>", header, "\< #\n\>"]\ ; \n
        WriteString[outstream, str1, "\<\n\>"]\ ; \n
        WriteString[outstream, "\<# ======== #\>"\ , "\<\n\>"]\ ; \n
        WriteString[outstream, "\<#| \>"<>"\<POINTS |#\n\>"\ ]\ ; \n
        WriteString[outstream, "\<# ======== #\n\n\>"]\ ; \n
        WriteString[outstream, int2string[npoints], 
          "\<  # Number of points #\n\n\>"]\ ; \n
        WriteString[outstream, "\<# Nodes which define the boundary #\n\>"]
          \ ; \nDo[\n\t
          WriteString[outstream, \n\t\tint2string[bpoints[\([k, 1]\)]\ ], 
            "\<: \>"\ , \n\t\ \ real2string[bpoints[\([k, 2, 1]\)]\ ], \ 
            "\< \>", \t\ \ \n\t\treal2string[bpoints[\([k, 2, 2]\)]\ ], \ 
            "\< \>", \ \n\t\ real2string[bpoints[\([k, 3]\)]], \ "\< \>", \n
            \ \ \ \ int2string[bpoints[\([k, 4]\)]], "\<\n\>"\n\t], 
          \n{k, 1, Length[bndry]}\n]\ ; \n
        WriteString[outstream, "\<\n# ========== #\>"\ , "\<\n\>"]\ ; \n
        WriteString[outstream, "\<#| \>"<>"\<SEGMENTS |#\n\>"\ ]\ ; \n
        WriteString[outstream, "\<# ========== #\n\n\>"]\ ; \n
        WriteString[outstream, \n\t\ \ \ \ ToString[nsegments], 
          "\<  # Number of segments #\n\n\>"]\ ; \n
        WriteString[outstream, "\<# Boundary segments #\n\>"]\ ; \n
        Do[\n\tWriteString[outstream, int2string[k - 1], "\<: \>"\ , \n\t\ \ 
            int2string[bndry[\([k, 1, 1]\)]\ ], \ "\< \>", \ \n\t\ \ 
            int2string[bndry[\([k, 1, 2]\)]\ ]\ , "\< \>", \n\ \ \ \ 
            int2string[bndry[\([k, 2]\)]], "\<\n\>"], 
          \n{k, 1, Length[bndry]}\n]\ ; \n
        Print["\<The data file '\>", filename, 
          "\<' has been created successfully.\>"]\ ; \nClose[outstream]\ ; \n]
        \t\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(assembleboundary\  := \ 
      Module[\n\t\t\ \ {pointsList, bpiece, k}, \n\t\tpPointer\  = \ 0\ ; \n
        Do[\n\t\ \ \ \ 
          pointsList[k] = 
            createPointsList[pts[k], raffinment[k], marker[k]]\ ; \n
          \ \ \ \ \ \ 
          bpiece[k]\  = createSegmentsList[pointsList[k], conditions[k]]\ , 
          \n{k, 1, npieces}]\ \ ; \n
        bpoints\  = \ 
          Flatten[Join[Table[pointsList[k], {k, 1, npieces}]], 1]\ ; \n
        bndry\  = \ Flatten[Join[Table[bpiece[k], {k, 1, npieces}]\ ], 1]\ ; 
        \n\t\tPrint[
          "\<List of points 'bpoints' has been successfully created.\>"]\ ; 
        Print["\<List of segments 'bndry' has been successfully created.\>"]
          \ ; \n]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(showboundary[bpoints_List, bndry_List]\  := \ 
      Module[\n\t\t\ \ \ {sgmntsList, k, textsList}, \n\t
        sgmntsList\  = \ \n\t\ 
          Table[\n\t\t\ 
            Line[{\ bpoints[\([bndry[\([k, 1, 1]\)] + 1, 2]\)], \n
                \t\t\t\t\ \ \ \ \ \ \ \ \ \ \ 
                bpoints[\([bndry[\([k, 1, 2]\)] + 1, 2]\)]}], 
            \ {k, 1, Length[bndry]}]\ ; \n
        textsList\  = \ \n\t\ \ 
          Table[Text[bpoints[\([k, 1]\)], bpoints[\([k, 2]\)]\ ], 
            \ \ \ \ \ \ \ \ \ \ \ \ \ \n\t\t\t{k, 1, Length[bpoints]}]\ ; \n
        Show[Graphics[{\n\t\tsgmntsList, \n\t\ttextsList\ \n\t\t}], \n
          AspectRatio -> Automatic]]\)], "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Some inputs:", "Section"],

Cell["\<\
Please give the name of outputfile for EasyMesh and a short header \
(the header will be wroten as a comment at the beginning of the \
datafile):\
\>", "Text"],

Cell[BoxData[
    \(filename\  = \ "\<test.d\>"\ ; \nheader\  = \ "\<test.d\>"; \)], "Input"],

Cell["\<\
Please give the exact number of the closed curves  which compose \
the complete boundary:\
\>", "Text"],

Cell[BoxData[
    \(\(\(npieces\  = \ 2; \)\ \)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Construction of the boundary", "Section"],

Cell["\<\
You have to define here the two closed curves which compose the \
boundary of your domain:\
\>", "Text"],

Cell[CellGroupData[{

Cell["First part of boundary (a rectangle):", "Subsection"],

Cell[BoxData[
    \(\(pts[1] = \ {{0, 0}, {2, 0}, {2, 1}, {0, 1}}\ ; \)\)], "Input"],

Cell[BoxData[
    \(\(conditions[1]\  = \ {2, 1, 2, 1}\ ; \)\)], "Input"],

Cell[BoxData[
    \(raffinment[1]\  = \ 0.05; \nmarker[1]\  = \ 1\ ; \)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["2nd part of boundary", "Subsection"],

Cell[BoxData[
    \(nphi\  = \ 20\ ; \ndphi\  = \ 2\ Pi/nphi\ ; \)], "Input"],

Cell[BoxData[
    \(\(pts[2] = \ 
      Table[{1\  + \ 0.3  Cos[\(-dphi\)\ k], \ 
          0.5\  + \ 0.3\ Sin[\(-dphi\)\ k]\ }, \ {k, 0, nphi - 1}]\ ; \)\)], 
  "Input"],

Cell[BoxData[
    \(\(conditions[2]\  = \ Table[2, \ {k, 0, nphi - 1}]\ ; \)\)], "Input"],

Cell[BoxData[
    \(raffinment[2]\  = \ 0.04; \nmarker[2]\  = \ 2\ ; \)], "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Assembling pieces of boundary", "Section"],

Cell[BoxData[
    \(assembleboundary\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Graphic of actual boundary", "Section"],

Cell[BoxData[
    \(showboundary[bpoints, bndry]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Generation of the datafile", "Section"],

Cell[BoxData[
    \(\(createfile[bpoints, bndry, filename, header]\ \)\)], "Input"]
}, Closed]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"X 3.0",
ScreenRectangle->{{0, 1152}, {0, 864}},
WindowSize->{703, 592},
WindowMargins->{{157, Automatic}, {Automatic, 21}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{597.562, 842.375},
"PaperOrientation"->"Portrait",
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 45, 0, 104, "Title"],
Cell[1779, 53, 51, 0, 27, "SmallText"],

Cell[CellGroupData[{
Cell[1855, 57, 350, 7, 89, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2230, 68, 44, 0, 52, "Section"],
Cell[2277, 70, 4395, 102, 1134, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6709, 177, 62, 1, 52, "Section",
  InitializationCell->True],
Cell[6774, 180, 81, 2, 27, "Input",
  InitializationCell->True],
Cell[6858, 184, 331, 6, 91, "Input",
  InitializationCell->True],
Cell[7192, 192, 524, 11, 123, "Input",
  InitializationCell->True],
Cell[7719, 205, 2523, 41, 731, "Input",
  InitializationCell->True],
Cell[10245, 248, 785, 16, 203, "Input",
  InitializationCell->True],
Cell[11033, 266, 727, 14, 235, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[11797, 285, 31, 0, 32, "Section"],
Cell[11831, 287, 169, 4, 50, "Text"],
Cell[12003, 293, 93, 1, 43, "Input"],
Cell[12099, 296, 113, 3, 32, "Text"],
Cell[12215, 301, 60, 1, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[12312, 307, 47, 0, 32, "Section"],
Cell[12362, 309, 114, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[12501, 316, 59, 0, 46, "Subsection"],
Cell[12563, 318, 84, 1, 27, "Input"],
Cell[12650, 321, 73, 1, 27, "Input"],
Cell[12726, 324, 82, 1, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12845, 330, 42, 0, 46, "Subsection"],
Cell[12890, 332, 77, 1, 43, "Input"],
Cell[12970, 335, 170, 4, 27, "Input"],
Cell[13143, 341, 89, 1, 27, "Input"],
Cell[13235, 344, 82, 1, 43, "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[13366, 351, 48, 0, 32, "Section"],
Cell[13417, 353, 49, 1, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[13503, 359, 45, 0, 32, "Section"],
Cell[13551, 361, 61, 1, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[13649, 367, 45, 0, 32, "Section"],
Cell[13697, 369, 83, 1, 27, "Input"]
}, Closed]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)


